//===-- temu-c/Bitmanip.h - Low level bit/byte manipulation -----*- C++ -*-===//
//
// T-EMU: The Terma Emulator
// (c) Terma 2015
// Authors: Mattias Holm <maho (at) terma.com>
//
//===----------------------------------------------------------------------===//

#ifndef TEMU_BITMANIP_H
#define TEMU_BITMANIP_H

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

// TODO: Make this platform independent

static inline uint16_t
temu_swap16(uint16_t HWord)
{
  uint16_t Res = HWord << 8 | HWord >> 8;
  return Res;
}

static inline uint32_t
temu_swap32(uint32_t Word)
{
  uint32_t Res = (((uint32_t)temu_swap16(Word)) << 16)
    | (uint32_t)temu_swap16(Word >> 16);
  return Res;
}

static inline uint64_t
temu_swap64(uint64_t DWord)
{
  uint64_t Res = (((uint64_t)temu_swap32(DWord)) << 32)
    | (uint64_t)temu_swap32(DWord >> 32);
  return Res;
}


static inline uint16_t
temu_swapBigHost16(uint16_t HWord)
{
  return temu_swap16(HWord);
}

static inline uint32_t
temu_swapBigHost32(uint32_t Word)
{
  return temu_swap32(Word);
}

static inline uint64_t
temu_swapBigHost64(uint64_t DWord)
{
  return temu_swap64(DWord);
}

static inline uint16_t
temu_swapLittleHost16(uint16_t HWord)
{
  return HWord;
}

static inline uint32_t
temu_swapLittleHost32(uint32_t Word)
{
  return Word;
}

static inline uint64_t
temu_swapLittleHost64(uint64_t DWord)
{
  return DWord;
}


// TODO: We should rename this header...
static inline int
temu_ctz32(uint32_t Word)
{
  if (Word == 0) return 32;
  int Res = __builtin_ctz(Word);
  return Res;
}

// TODO: We should rename this header...
static inline int
temu_clz32(uint32_t Word)
{
  if (Word == 0) return 32;
  int Res = __builtin_clz(Word);
  return Res;
}

// TODO: We should rename this header...
static inline int
temu_popcount32(uint32_t Word)
{
  int Res = __builtin_popcount(Word);
  return Res;
}

static inline int
temu_parity32(uint32_t Word)
{
  int Res = __builtin_parity(Word);
  return Res;
}

static inline int
temu_ctz64(uint64_t Word)
{
  if (Word == 0) return 64;
  int Res = __builtin_ctzl(Word);
  return Res;
}

// TODO: We should rename this header...
static inline int
temu_clz64(uint64_t Word)
{
  if (Word == 0) return 64;
  int Res = __builtin_clzl(Word);
  return Res;
}

// TODO: We should rename this header...
static inline int
temu_popcount64(uint64_t Word)
{
  int Res = __builtin_popcountl(Word);
  return Res;
}

static inline int
temu_parity64(uint64_t Word)
{
  int Res = __builtin_parityl(Word);
  return Res;
}


#ifdef __cplusplus
}
#endif

#endif /* !TEMU_BITMANIP_H */
